/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2017 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.blocks.cable;

import net.minecraft.block.BlockContainer;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.properties.PropertyBool;
import net.minecraft.block.properties.PropertyEnum;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumBlockRenderType;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.NonNullList;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.RayTraceResult;
import net.minecraft.world.ChunkCache;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraft.world.chunk.Chunk;
import net.minecraftforge.energy.CapabilityEnergy;
import techreborn.client.TechRebornCreativeTab;
import techreborn.init.ModBlocks;
import techreborn.tiles.cable.TileCable;

import javax.annotation.Nullable;
import java.security.InvalidParameterException;

/**
 * Created by modmuss50 on 19/05/2017.
 */
public class BlockCable extends BlockContainer {

	public static final PropertyBool EAST = PropertyBool.func_177716_a("east");
	public static final PropertyBool WEST = PropertyBool.func_177716_a("west");
	public static final PropertyBool NORTH = PropertyBool.func_177716_a("north");
	public static final PropertyBool SOUTH = PropertyBool.func_177716_a("south");
	public static final PropertyBool UP = PropertyBool.func_177716_a("up");
	public static final PropertyBool DOWN = PropertyBool.func_177716_a("down");
	public static final IProperty<EnumCableType> TYPE = PropertyEnum.func_177709_a("type", EnumCableType.class);

	public BlockCable() {
		super(Material.field_151576_e);
		func_149647_a(TechRebornCreativeTab.instance);
		this.func_149663_c("techreborn.cable");
		func_180632_j(func_176223_P().func_177226_a(EAST, false).func_177226_a(WEST, false).func_177226_a(NORTH, false).func_177226_a(SOUTH, false).func_177226_a(UP, false).func_177226_a(DOWN, false).func_177226_a(TYPE, EnumCableType.COPPER));
	}

	public static ItemStack getCableByName(String name, int count) {
		for (int i = 0; i < EnumCableType.values().length; i++) {
			if (EnumCableType.values()[i].func_176610_l().equalsIgnoreCase(name)) {
				return new ItemStack(ModBlocks.CABLE,
					count, i);
			}
		}
		throw new InvalidParameterException("The cable " + name + " could not be found.");
	}

	public static ItemStack getCableByName(String name) {
		return getCableByName(name, 1);
	}

	@Override
	public ItemStack getPickBlock(IBlockState state, RayTraceResult target, World world, BlockPos pos, EntityPlayer player) {
		return new ItemStack(this, 1, 0);
	}

	@Override
	public int func_180651_a(IBlockState state) {
		return func_176201_c(func_176223_P());
	}

	@Override
	public boolean func_149662_c(IBlockState state) {
		return false;
	}

	@Override
	public boolean func_149730_j(IBlockState state) {
		return false;
	}

	@Override
	protected BlockStateContainer func_180661_e() {
		return new BlockStateContainer(this, EAST, WEST, NORTH, SOUTH, UP, DOWN, TYPE);
	}

	@Override
	public IBlockState getStateForPlacement(World world, BlockPos pos, EnumFacing facing, float hitX, float hitY, float hitZ, int meta, EntityLivingBase placer, EnumHand hand) {
		return func_176203_a(placer.func_184586_b(hand).func_77952_i());
	}

	@Override
	public void func_149666_a(CreativeTabs tab, NonNullList<ItemStack> list) {
		for (EnumCableType cableType : EnumCableType.values()) {
			list.add(new ItemStack(this, 1, cableType.ordinal()));
		}
	}

	@Override
	public int func_176201_c(IBlockState state) {
		return state.func_177229_b(TYPE).ordinal();
	}

	@Override
	public IBlockState func_176203_a(int meta) {
		return func_176223_P().func_177226_a(TYPE, EnumCableType.values()[meta]);
	}

	@Override
	public boolean func_149686_d(IBlockState state) {
		return false;
	}

	public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess source, BlockPos pos) {
		state = state.func_185899_b(source, pos);
		float minX = state.func_177229_b(WEST) ? 0.0F : 0.3125F;
		float minY = state.func_177229_b(DOWN) ? 0.0F : 0.3125F;
		float minZ = state.func_177229_b(NORTH) ? 0.0F : 0.3125F;
		float maxX = state.func_177229_b(EAST) ? 1.0F : 0.6875F;
		float maxY = state.func_177229_b(UP) ? 1.0F : 0.6875F;
		float maxZ = state.func_177229_b(SOUTH) ? 1.0F : 0.6875F;
		return new AxisAlignedBB((double) minX, (double) minY, (double) minZ, (double) maxX, (double) maxY, (double) maxZ);
	}

	@Override
	public IBlockState func_176221_a(IBlockState state, IBlockAccess worldIn, BlockPos pos) {
		IBlockState actualState = state;
		for (EnumFacing facing : EnumFacing.values()) {
			TileEntity tileEntity = getTileEntitySafely(worldIn, pos.func_177972_a(facing));
			if (tileEntity != null) {
				actualState = actualState.func_177226_a(getProperty(facing), tileEntity.hasCapability(CapabilityEnergy.ENERGY, facing.func_176734_d()));
			}
		}
		return actualState;
	}

	//see for more info https://www.reddit.com/r/feedthebeast/comments/5mxwq9/psa_mod_devs_do_you_call_worldgettileentity_from/
	public TileEntity getTileEntitySafely(IBlockAccess blockAccess, BlockPos pos) {
		if (blockAccess instanceof ChunkCache) {
			return ((ChunkCache) blockAccess).func_190300_a(pos, Chunk.EnumCreateEntityType.CHECK);
		} else {
			return blockAccess.func_175625_s(pos);
		}
	}

	public IProperty<Boolean> getProperty(EnumFacing facing) {
		switch (facing) {
			case EAST:
				return EAST;
			case WEST:
				return WEST;
			case NORTH:
				return NORTH;
			case SOUTH:
				return SOUTH;
			case UP:
				return UP;
			case DOWN:
				return DOWN;
			default:
				return EAST;
		}
	}

	@Nullable
	@Override
	public TileEntity func_149915_a(World worldIn, int meta) {
		return new TileCable();
	}

	@Override
	public EnumBlockRenderType func_149645_b(IBlockState state) {
		return EnumBlockRenderType.MODEL;
	}

}
