/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2017 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.client.container.builder.slot;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.inventory.Slot;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import reborncore.api.tile.IUpgrade;
import reborncore.api.tile.IUpgradeable;
import reborncore.common.util.Inventory;
import techreborn.client.container.IRightClickHandler;
import techreborn.client.container.builder.BuiltContainer;

import java.util.function.Predicate;

public class UpgradeSlot extends Slot implements IRightClickHandler {

	private Predicate<ItemStack> filter;

	public UpgradeSlot(final IInventory inventory, final int index, final int xPosition, final int yPosition) {
		super(inventory, index, xPosition, yPosition);
	}

	@Override
	public boolean func_75214_a(final ItemStack stack) {
		return stack.func_77973_b() instanceof IUpgrade;
	}

	@Override
	public int func_75219_a() {
		return 1;
	}

	@Override
	public boolean handleRightClick(int slotID, EntityPlayer player, BuiltContainer container) {
		if (field_75224_c instanceof Inventory) {
			Inventory inv = (Inventory) field_75224_c;
			TileEntity tileEntity = inv.getTileBase();
			if (tileEntity instanceof IUpgradeable) {
				IUpgradeable upgradeable = (IUpgradeable) tileEntity;
				if (upgradeable.canBeUpgraded()) {
					ItemStack stack = upgradeable.getUpgradeInvetory().func_70301_a(slotID);
					if (!stack.func_190926_b() && stack.func_77973_b() instanceof IUpgrade) {
						if (player.field_70170_p.field_72995_K) {
							((IUpgrade) stack.func_77973_b()).handleRightClick(tileEntity, stack, container, slotID);
						}
					}
				}
			}
		}

		return true;
	}
}
